with Ada.Text_IO; use  Ada.Text_IO;
with Interfaces.Fortran; use  Interfaces.Fortran;
with lapack;

Procedure tcheev is

	use lapack.Integer_IO;
	use lapack.Real_IO;

	N	 : Fortran_Integer := 3;
	A,X	 : lapack.Fortran_Complex_Matrix ( 1..N, 1..N);
	W	 : lapack.Fortran_Real_Vector ( 1..N );
	LWORK	 : Fortran_Integer := 2*N-1;
	WORK	 : lapack.Fortran_Complex_Vector ( 1..LWORK );
	RWORK	 : lapack.Fortran_Real_Vector ( 1..3*N-2 );
	INFO	 : Fortran_Integer;

	TMP : Complex := (0.0,0.0);
	ERR : Real := 0.0;

begin
        -- A is Hermitian matrix
        A := (( ( 3.0, 0.0), ( 1.0,-2.0), ( 4.0, 7.0)),
              ( ( 1.0,+2.0), (-4.0, 0.0), ( 0.0,-2.0)),
              ( ( 4.0,-7.0), ( 0.0, 2.0), ( 2.0, 0.0)));

	X := A;

	lapack.CHEEV (
	JOBZ	 => 'V',
	UPLO	 => 'U',
	N	 => N,
	A	 => A,
	LDA	 => N,
	W	 => W,
	WORK	 => WORK,
	LWORK	 => LWORK,
	RWORK	 => RWORK,
	INFO	 => INFO);

	-- Test if error
	if (INFO /= 0) then
	   Put ("ERROR ");
	   Put ( INFO );
	   New_line;
	end if;

	-- Verify routine
         ERR := 0.0;

        for I in W'Range(1) loop

          for J in X'Range(1) loop
            TMP := (0.0,0.0);

            for K in X'Range(2) loop
                TMP := TMP + ( X(J,K) * A(K,I) ) ;
            end loop;

            TMP := TMP - ( W(I) * A(J,I) );
            -- PUT (TMP);
            -- new_line;
            ERR := ERR + abs( TMP );
          end loop;
          
        end loop;

	PUT ( "Abs Error:");
	PUT ( ERR );
	New_line;

end tcheev;
