with Ada.Text_IO; use  Ada.Text_IO;
with Interfaces.Fortran; use  Interfaces.Fortran;
with lapack;

Procedure tssbev is

	use lapack.Integer_IO;
	use lapack.Real_IO;

	N	 : Fortran_Integer := 3;
	KD	 : Fortran_Integer := 1;

	LDAB	 : Fortran_Integer := KD + 1;
	AB	 : lapack.Fortran_Real_Matrix ( 1..LDAB, 1..N);

	W	 : lapack.Fortran_Real_Vector ( 1..N );
	Z,A	 : lapack.Fortran_Real_Matrix ( 1..N, 1..N);
	WORK	 : lapack.Fortran_Real_Vector ( 1..3*N -2);
	INFO	 : Fortran_Integer;

	ERR, TMP : Real := 0.0;

begin
        A := ((2.0,  1.0,  0.0),
              (1.0,  2.0, -1.0),
              (0.0, -1.0,  6.0));

        -- AB is A in band storage scheme
        AB := (( 0.0  ,  A(1,2), A(2,3)),
               (A(1,1),  A(2,2), A(3,3)));
        

	lapack.SSBEV (
	JOBZ	 => 'V',
	UPLO	 => 'U',
	N	 => N,
	KD	 => KD,
	AB	 => AB,
	LDAB	 => LDAB,
	W	 => W,
	Z	 => Z,
	LDZ	 => N,
	WORK	 => WORK,
	INFO	 => INFO);

	-- Test if error
	if (INFO /= 0) then
	   Put ("ERROR ");
	   Put ( INFO );
	   New_line;
	end if;

	-- Verify routine
        ERR := 0.0;

        for I in W'Range(1) loop

          for J in A'Range(1) loop
            TMP := 0.0;

            for K in A'Range(2) loop
                TMP := TMP + ( A(J,K) * Z(K,I) ) ;
            end loop;

            TMP := TMP - ( W(I) * Z(J,I) );
            -- PUT (TMP);
            -- new_line;
            ERR := ERR + abs( TMP );
          end loop;
          
        end loop;
                

	PUT ( "Abs Error:");
	PUT ( ERR );
	New_line;

end tssbev;
